@@ -1,6 +1,39 @@
 #include "parallel_qr.h"
 #include "LAPACK_wrappers.h"
 #include <algorithm>
+#include "constant.h"
+
+namespace {
+/* handles working memory */
+static size_t wk_mem_per_thread  = 0L,
+              current_wk_size    = 0L;
+static std::unique_ptr<double[]> current_wk_mem =
+  std::unique_ptr<double[]>();
+
+double * get_working_memory(thread_pool const &pool){
+  size_t const my_num = pool.get_id();
+  return current_wk_mem.get() + my_num * wk_mem_per_thread;
+}
+} // namespace
+
+void set_p_qr_working_memory(size_t const max_m, size_t const max_n,
+                             size_t const n_threads){
+  constexpr size_t const mult = cacheline_size() / sizeof(double),
+                     min_size = 2L * mult;
+  size_t m_dim = get_qr_mem_size(max_m, max_n);
+  m_dim = std::max(m_dim, min_size);
+  m_dim = (m_dim + mult - 1L) / mult;
+  m_dim *= mult;
+  wk_mem_per_thread = m_dim;
+
+  size_t const n_t = std::max(n_threads, static_cast<size_t>(1L)) + 1L,
+          new_size = n_t * m_dim;
+  if(new_size > current_wk_size){
+    current_wk_mem.reset(new double[new_size]);
+    current_wk_size = new_size;
+
+  }
+}
 
 arma::mat R_F::R_rev_piv() const {
   arma::uvec piv = pivot;
@@ -9,22 +42,23 @@ arma::mat R_F::R_rev_piv() const {
 }
 
 qr_parallel::worker::worker
-  (std::unique_ptr<qr_data_generator> generator):
-  my_generator(std::move(generator)) {}
+  (std::unique_ptr<qr_data_generator> generator,
+   thread_pool &th_pool):
+  my_generator(std::move(generator)), th_pool(th_pool) {}
 
 R_F qr_parallel::worker::operator()(){
   qr_work_chunk my_chunk = my_generator->get_chunk();
-  QR_factorization qr(my_chunk.X);
+  QR_base qr(my_chunk.X, get_working_memory(th_pool));
   const unsigned n_rows =
     std::min(my_chunk.X.n_cols - 1, my_chunk.Y.n_rows - 1);
-  arma::mat F = qr.qy(my_chunk.Y, true).rows(0, n_rows);
+  arma::mat F = qr.qyt(my_chunk.Y).rows(0, n_rows);
 
   return R_F { qr.R(), qr.pivot(), std::move(F), my_chunk.dev };
 }
 
 qr_parallel::qr_parallel(
   ptr_vec generators, const unsigned int max_threads):
-  n_threads(std::max((unsigned int)1L, max_threads)),
+  n_threads(std::max(static_cast<unsigned>(1L), max_threads)),
   futures(), th_pool(n_threads)
   {
     while(!generators.empty()){
@@ -34,7 +68,7 @@ qr_parallel::qr_parallel(
   }
 
 void qr_parallel::submit(std::unique_ptr<qr_data_generator> generator){
-  futures.push_back(th_pool.submit(worker(std::move(generator))));
+  futures.push_back(th_pool.submit(worker(std::move(generator), th_pool)));
 }
 
 qr_parallel::get_stacks_res_obj qr_parallel::get_stacks_res(){
@@ -53,7 +87,7 @@ qr_parallel::get_stacks_res_obj qr_parallel::get_stacks_res(){
 
     /* we assume that the first tasks are done first */
     for(arma::uword j = 0; f != futures.end() and j < n_threads; ++j, ++f){
-      if(f->wait_for(std::chrono::milliseconds(10)) ==
+      if(f->wait_for(std::chrono::microseconds(1)) ==
          std::future_status::ready){
         R_F R_Fs_i = f->get();
         if(is_first){
@@ -87,8 +121,8 @@ R_F qr_parallel::compute(){
   auto stacked = get_stacks_res();
 
   /* make new QR decomp and compute new F */
-  QR_factorization qr(stacked.R_stack);
-  arma::mat F = qr.qy(stacked.F_stack, true).rows(0, stacked.p - 1);
+  QR_base qr(stacked.R_stack, current_wk_mem.get());
+  arma::mat F = qr.qyt(stacked.F_stack).rows(0, stacked.p - 1);
 
   return { qr.R(), qr.pivot(), std::move(F), stacked.dev };
 }
@@ -96,7 +130,8 @@ R_F qr_parallel::compute(){
 qr_dqrls_res qr_parallel::compute_dqrls(const double tol){
   auto stacked = get_stacks_res();
 
-  arma::vec y(stacked.F_stack.memptr(), stacked.F_stack.n_elem, false);
+  arma::vec y(stacked.F_stack.memptr(), stacked.F_stack.n_elem,
+              false, true);
   auto o = dqrls_wrap(stacked.R_stack, y, tol);
 
   arma::uword di = std::min(o.qr.n_cols, o.qr.n_rows) - 1L;
